#
# Autogenerated by Thrift Compiler (0.22.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec
from uuid import UUID

import sys
from .ttypes import *
DISPNULL = 0
DISPTIME = 1
DISPXY = 2
DISPPSD = 4
DISPSPEC = 8
DISPRAST = 16
DISPOPTCPLX = 256
DISPOPTLOG = 512
DISPOPTSTEM = 1024
DISPOPTSTRIP = 2048
DISPOPTSCATTER = 4096
