#
# Copyright 2005,2006,2007 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
"""
Legacy utilities for packet modulation
"""

default_access_code_binary = b'\xAC\xDD\xA4\xE2\xF2\x8C\x20\xFC'
# This is the same as default_access_code_binary, but as a string of 1s and 0s
default_access_code = "1010110011011101101001001110001011110010100011000010000011111100"
